@@ -31,14 +31,17 @@ impl Search {
 
   /// Find justfile given search configuration and invocation directory
   pub(crate) fn find(
-    search_config: &SearchConfig,
+    ceiling: Option<&Path>,
     invocation_directory: &Path,
+    search_config: &SearchConfig,
   ) -> SearchResult<Self> {
     match search_config {
-      SearchConfig::FromInvocationDirectory => Self::find_in_directory(invocation_directory),
+      SearchConfig::FromInvocationDirectory => {
+        Self::find_in_directory(ceiling, invocation_directory)
+      }
       SearchConfig::FromSearchDirectory { search_directory } => {
         let search_directory = Self::clean(invocation_directory, search_directory);
-        let justfile = Self::justfile(&search_directory)?;
+        let justfile = Self::justfile(ceiling, &search_directory)?;
         let working_directory = Self::working_directory_from_justfile(&justfile)?;
         Ok(Self {
           justfile,
@@ -47,7 +50,7 @@ impl Search {
       }
       SearchConfig::GlobalJustfile => Ok(Self {
         justfile: Self::find_global_justfile()?,
-        working_directory: Self::project_root(invocation_directory)?,
+        working_directory: Self::project_root(ceiling, invocation_directory)?,
       }),
       SearchConfig::WithJustfile { justfile } => {
         let justfile = Self::clean(invocation_directory, justfile);
@@ -88,7 +91,7 @@ impl Search {
   }
 
   /// Find justfile starting from parent directory of current justfile
-  pub(crate) fn search_parent_directory(&self) -> SearchResult<Self> {
+  pub(crate) fn search_parent_directory(&self, ceiling: Option<&Path>) -> SearchResult<Self> {
     let parent = self
       .justfile
       .parent()
@@ -96,12 +99,12 @@ impl Search {
       .ok_or_else(|| SearchError::JustfileHadNoParent {
         path: self.justfile.clone(),
       })?;
-    Self::find_in_directory(parent)
+    Self::find_in_directory(ceiling, parent)
   }
 
   /// Find justfile starting in given directory searching upwards in directory tree
-  fn find_in_directory(starting_dir: &Path) -> SearchResult<Self> {
-    let justfile = Self::justfile(starting_dir)?;
+  fn find_in_directory(ceiling: Option<&Path>, starting_dir: &Path) -> SearchResult<Self> {
+    let justfile = Self::justfile(ceiling, starting_dir)?;
     let working_directory = Self::working_directory_from_justfile(&justfile)?;
     Ok(Self {
       justfile,
@@ -113,10 +116,11 @@ impl Search {
   pub(crate) fn init(
     search_config: &SearchConfig,
     invocation_directory: &Path,
+    ceiling: Option<&Path>,
   ) -> SearchResult<Self> {
     match search_config {
       SearchConfig::FromInvocationDirectory => {
-        let working_directory = Self::project_root(invocation_directory)?;
+        let working_directory = Self::project_root(ceiling, invocation_directory)?;
         let justfile = working_directory.join(DEFAULT_JUSTFILE_NAME);
         Ok(Self {
           justfile,
@@ -125,7 +129,7 @@ impl Search {
       }
       SearchConfig::FromSearchDirectory { search_directory } => {
         let search_directory = Self::clean(invocation_directory, search_directory);
-        let working_directory = Self::project_root(&search_directory)?;
+        let working_directory = Self::project_root(ceiling, &search_directory)?;
         let justfile = working_directory.join(DEFAULT_JUSTFILE_NAME);
         Ok(Self {
           justfile,
@@ -153,7 +157,7 @@ impl Search {
 
   /// Search upwards from `directory` for a file whose name matches one of
   /// `JUSTFILE_NAMES`
-  fn justfile(directory: &Path) -> SearchResult<PathBuf> {
+  fn justfile(ceiling: Option<&Path>, directory: &Path) -> SearchResult<PathBuf> {
     for directory in directory.ancestors() {
       let mut candidates = BTreeSet::new();
 
@@ -181,6 +185,12 @@ impl Search {
         1 => return Ok(candidates.into_iter().next().unwrap()),
         _ => return Err(SearchError::MultipleCandidates { candidates }),
       }
+
+      if let Some(ceiling) = ceiling {
+        if directory == ceiling {
+          break;
+        }
+      }
     }
 
     Err(SearchError::NotFound)
@@ -207,7 +217,7 @@ impl Search {
   /// Search upwards from `directory` for the root directory of a software
   /// project, as determined by the presence of one of the version control
   /// system directories given in `PROJECT_ROOT_CHILDREN`
-  fn project_root(directory: &Path) -> SearchResult<PathBuf> {
+  fn project_root(ceiling: Option<&Path>, directory: &Path) -> SearchResult<PathBuf> {
     for directory in directory.ancestors() {
       let entries = fs::read_dir(directory).map_err(|io_error| SearchError::Io {
         io_error,
@@ -225,6 +235,12 @@ impl Search {
           }
         }
       }
+
+      if let Some(ceiling) = ceiling {
+        if directory == ceiling {
+          break;
+        }
+      }
     }
 
     Ok(directory.to_owned())
@@ -250,7 +266,7 @@ mod tests {
   #[test]
   fn not_found() {
     let tmp = testing::tempdir();
-    match Search::justfile(tmp.path()) {
+    match Search::justfile(None, tmp.path()) {
       Err(SearchError::NotFound) => {}
       _ => panic!("No justfile found error was expected"),
     }
@@ -270,7 +286,7 @@ mod tests {
     }
     fs::write(&path, "default:\n\techo ok").unwrap();
     path.pop();
-    match Search::justfile(path.as_path()) {
+    match Search::justfile(None, path.as_path()) {
       Err(SearchError::MultipleCandidates { .. }) => {}
       _ => panic!("Multiple candidates error was expected"),
     }
@@ -283,7 +299,7 @@ mod tests {
     path.push(DEFAULT_JUSTFILE_NAME);
     fs::write(&path, "default:\n\techo ok").unwrap();
     path.pop();
-    if let Err(err) = Search::justfile(path.as_path()) {
+    if let Err(err) = Search::justfile(None, path.as_path()) {
       panic!("No errors were expected: {err}");
     }
   }
@@ -306,7 +322,7 @@ mod tests {
     path.push(spongebob_case);
     fs::write(&path, "default:\n\techo ok").unwrap();
     path.pop();
-    if let Err(err) = Search::justfile(path.as_path()) {
+    if let Err(err) = Search::justfile(None, path.as_path()) {
       panic!("No errors were expected: {err}");
     }
   }
@@ -322,7 +338,7 @@ mod tests {
     fs::create_dir(&path).expect("test justfile search: failed to create intermediary directory");
     path.push("b");
     fs::create_dir(&path).expect("test justfile search: failed to create intermediary directory");
-    if let Err(err) = Search::justfile(path.as_path()) {
+    if let Err(err) = Search::justfile(None, path.as_path()) {
       panic!("No errors were expected: {err}");
     }
   }
@@ -341,7 +357,7 @@ mod tests {
     path.pop();
     path.push("b");
     fs::create_dir(&path).expect("test justfile search: failed to create intermediary directory");
-    match Search::justfile(path.as_path()) {
+    match Search::justfile(None, path.as_path()) {
       Ok(found_path) => {
         path.pop();
         path.push(DEFAULT_JUSTFILE_NAME);
@@ -370,7 +386,7 @@ mod tests {
 
     let search_config = SearchConfig::FromInvocationDirectory;
 
-    let search = Search::find(&search_config, &sub).unwrap();
+    let search = Search::find(None, &sub, &search_config).unwrap();
 
     assert_eq!(search.justfile, justfile);
     assert_eq!(search.working_directory, sub);
