@@ -12,7 +12,7 @@ fn convert_native_path(path: &Path) -> String {
 #[cfg(windows)]
 fn convert_native_path(path: &Path) -> String {
   // Translate path from windows style to unix style
-  let mut cygpath = Command::new("cygpath");
+  let mut cygpath = Command::new(env::var_os("JUST_CYGPATH").unwrap_or("cygpath".into()));
   cygpath.arg("--unix");
   cygpath.arg(path);
 
@@ -62,36 +62,30 @@ fn test_invocation_directory() {
 
   let status = output.status.code().unwrap();
   if status != expected_status {
-    println!("bad status: {} != {}", status, expected_status);
+    println!("bad status: {status} != {expected_status}");
     failure = true;
   }
 
   let stdout = str::from_utf8(&output.stdout).unwrap();
   if stdout != expected_stdout {
-    println!(
-      "bad stdout:\ngot:\n{:?}\n\nexpected:\n{:?}",
-      stdout, expected_stdout
-    );
+    println!("bad stdout:\ngot:\n{stdout:?}\n\nexpected:\n{expected_stdout:?}");
     failure = true;
   }
 
   let stderr = str::from_utf8(&output.stderr).unwrap();
   if stderr != expected_stderr {
-    println!(
-      "bad stderr:\ngot:\n{:?}\n\nexpected:\n{:?}",
-      stderr, expected_stderr
-    );
+    println!("bad stderr:\ngot:\n{stderr:?}\n\nexpected:\n{expected_stderr:?}");
     failure = true;
   }
 
-  if failure {
-    panic!("test failed");
-  }
+  assert!(!failure, "test failed");
 }
 
 #[test]
 fn invocation_directory_native() {
-  let Output { stdout, tempdir } = Test::new()
+  let Output {
+    stdout, tempdir, ..
+  } = Test::new()
     .justfile("x := invocation_directory_native()")
     .args(["--evaluate", "x"])
     .stdout_regex(".*")
